﻿#version 330 core

uniform sampler2D renderTexture;
uniform float globalTime = 0;
uniform float noiseAmount = 0;
uniform float fade = 0;

uniform float brightness = 1.0;
uniform float saturation = 1.0;
uniform float contrast = 1.2;

in vec2 uv;

out vec4 outColor;

//This produces a pseudo-random number between 0 and 1
//the sin and globalTime generates movement
//the fix numbers are used as seed
float rand(vec2 uv)
{
	return fract( (sin(dot(uv.xy, vec2(12.12 * globalTime, 67.67 * globalTime))) * 12345.12345 + globalTime));
}

float vignette(vec2 coord, float startFadeOut, float endFadeOut)
{
	float dist = distance(vec2(0.5), coord);
	return 1.0 - smoothstep(startFadeOut, endFadeOut, dist);
}

vec3 addBrightness(vec3 col)
{
	//simply multiply brightness to color
	return col * brightness;
}

vec3 adjustContrast(vec3 col)
{
	return col * contrast + vec3((0.5 - contrast * 0.5));
}

vec3 adjustSaturation(vec3 col)
{
	//luminance coefficient to get greyscale image
	vec3 lumCoeff = vec3(0.2125, 0.7154, 0.0721);

	//difference from color and luminance
	float intens = dot(col, lumCoeff);

	return mix(vec3(intens), col, saturation);
}

void main() {
	vec3 color = texture(renderTexture, uv).rgb;
	
	float startFadeOut = 0.3;
	float smoothness = 0.5;
	float vignette = vignette(uv, startFadeOut, startFadeOut + smoothness);

	//add vignette
	color *= vignette;
	//add noise
	color += vec3(rand(uv)) * noiseAmount;

	color = addBrightness(color);
	color = adjustContrast(color);
	color = adjustSaturation(color);

	//fading
	color = mix(color, vec3(0), fade);

	outColor = vec4(color, 1.0);
}