﻿#version 330 core
//Tutorial: https://learnopengl.com/Lighting/Basic-Lighting

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition_modelspace;
layout(location = 1) in vec3 vertexNormal_modelspace;
layout(location = 2) in vec2 texCoord;

//infos given to the fragment shader
out vec3 vertex_normal;
out vec3 vertex_worldPos;
out vec2 uv;
out vec4 lightSpacePos;

//uniforms 
uniform mat4 mvpMatrix;
uniform mat4 M;
uniform mat4 lightSpaceMatrix;

void main()
{
	vertex_worldPos = vec3( M * vec4(vertexPosition_modelspace, 1));	//vertex position in world space, for our calculations
	vertex_normal = mat3(transpose(inverse(M))) * vertexNormal_modelspace;	//normal matrix; when model is scaled; 'the transpose of the inverse of the upper-left corner of the model matrix'
	uv = vec2(texCoord.x, -texCoord.y);	//set texcoord for fragment shader, negate v component, different mapping of opengl and directX (https://stackoverflow.com/questions/44021587/opengl-texture-distortion-on-3d-model)

	lightSpacePos = lightSpaceMatrix * M * vec4(vertexPosition_modelspace, 1);

	// Output position of the vertex, in clip space : MVP * position
	gl_Position = mvpMatrix * vec4(vertexPosition_modelspace, 1);
}